@@ -1,15 +1,16 @@
 # -*- coding: utf-8 -*-
-import re
 import sys
-from typing import Iterable, Iterator, Optional, Pattern, Sequence, Tuple
+from typing import Iterable, Iterator, Optional, Sequence, Tuple
 
 import grpc
+from grpc import RpcError
 
 from esdbclient.esdbapi import (
     Streams,
     SubscriptionReadRequest,
     SubscriptionReadResponse,
     Subscriptions,
+    handle_rpc_error,
 )
 from esdbclient.events import NewEvent, RecordedEvent
 from esdbclient.exceptions import StreamNotFound
@@ -66,7 +67,7 @@ class EsdbClient:
 
     def read_all_events(
         self,
-        position: Optional[int] = None,
+        commit_position: Optional[int] = None,
         backwards: bool = False,
         filter_exclude: Sequence[str] = (ESDB_EVENTS_REGEX,),
         filter_include: Sequence[str] = (),
@@ -77,7 +78,7 @@ class EsdbClient:
         Reads recorded events in "all streams" in the database.
         """
         return self.streams.read(
-            commit_position=position,
+            commit_position=commit_position,
             backwards=backwards,
             filter_exclude=filter_exclude,
             filter_include=filter_include,
@@ -128,31 +129,32 @@ class EsdbClient:
 
     def subscribe_all_events(
         self,
-        position: Optional[int] = None,
+        commit_position: Optional[int] = None,
         filter_exclude: Sequence[str] = (ESDB_EVENTS_REGEX,),
         filter_include: Sequence[str] = (),
         timeout: Optional[float] = None,
-    ) -> Iterable[RecordedEvent]:
+    ) -> Iterator[RecordedEvent]:
         """
         Returns a catch-up subscription, from which recorded
         events in "all streams" in the database can be received.
         """
-        event_generator = self.streams.read(
-            commit_position=position,
+        read_resp = self.streams.read(
+            commit_position=commit_position,
+            filter_exclude=filter_exclude,
+            filter_include=filter_include,
             subscribe=True,
             timeout=timeout,
         )
-        return CatchupSubscription(
-            event_generator=event_generator,
-            filter_exclude=filter_exclude,
-            filter_include=filter_include,
-        )
+        return CatchupSubscription(read_resp=read_resp)
 
     def create_subscription(
-        self, group_name: str, from_end: bool = False, position: Optional[int] = None
+        self,
+        group_name: str,
+        from_end: bool = False,
+        commit_position: Optional[int] = None,
     ) -> None:
         self.subscriptions.create(
-            group_name=group_name, from_end=from_end, commit_position=position
+            group_name=group_name, from_end=from_end, commit_position=commit_position
         )
 
     def read_subscription(
@@ -161,33 +163,22 @@ class EsdbClient:
         return self.subscriptions.read(group_name=group_name)
 
 
-class CatchupSubscription:
+class CatchupSubscription(Iterator[RecordedEvent]):
     """
-    Encapsulates generator of recorded events when
-    reading events in a catch-up subscription.
-
-    Filters recorded events, because the server doesn't.
+    Encapsulates read response for a catch-up subscription.
     """
 
     def __init__(
         self,
-        event_generator: Iterable[RecordedEvent],
-        filter_exclude: Sequence[str] = (),
-        filter_include: Sequence[str] = (),
+        read_resp: Iterable[RecordedEvent],
     ):
-        self.event_generator = event_generator
-        if filter_exclude or filter_include:
-            if filter_include:
-                filter_regex = "^" + "|".join(filter_include) + "$"
-            else:
-                filter_regex = "^(?!(" + "|".join(filter_exclude) + ")).*$"
-            self.filter_regex: Optional[Pattern[str]] = re.compile(filter_regex)
-        else:
-            self.filter_regex = None
+        self.read_resp = iter(read_resp)
 
     def __iter__(self) -> Iterator[RecordedEvent]:
-        for event in self.event_generator:
-            if self.filter_regex is None:
-                yield event
-            elif re.match(self.filter_regex, event.type):
-                yield event
+        return self
+
+    def __next__(self) -> RecordedEvent:
+        try:
+            return next(self.read_resp)
+        except RpcError as e:
+            raise handle_rpc_error(e) from e  # pragma: no cover
